########################
#
# Sips (Freundlich-Langmuir) isotherm model This script is made
# freely available and this notice should be included with the
# distribution of this script.  This script may be freely distributed
# and modified.  Jeff Kenvin, December 2013
#
########################
import numpy as np
import mic
import scipy.optimize as optim
#
########
#
## data from sample file
p_ads, q_ads, n_ads, warm_fs, cold_fs, mass, desc = mic.isotherm('abs')
#csa, hsd, dcf, mol_weight, analysis_gas = mic.adsorptive_data(0)
#
## the n_ads is the number of adsorption points
p_ads = p_ads[0 : n_ads]            # only adsorption points for relative pressure
q_ads = q_ads[0 : n_ads]            # only adsorption points for volume adsorbed
#
# function to convert to mbar
def fmmHg2mbar( p ):
    pmBar = p * 1013.25 / 760.0
    return pmBar
# function to convert to mmoles
def fV2mmol( v ):
    nmmol = v/22.414
    return nmmol
#
x = fmmHg2mbar(p_ads)
xMin = np.min(x)
xMax = np.max(x)
xStep = (xMax-xMin)/100.0
xF = np.arange(xMin,xMax,xStep)
# convert to milli moles
y = fV2mmol(q_ads) 
#
# function to minimize for calc of L-F params -> n = nm*(bx)^g/(1+(bx)^g)
def fMinLF( par, x, y ):
    nm = par[0]
    b = par[1]
    t = par[2]
    err = y*(1.0 + (b*x)**t) - nm*(b*x)**t
    rmse = np.sqrt(np.sum(err**2))
    return rmse
#
# function to calculate iso from LF params
#
def fLF(par,x):
    nm = par[0]
    b = par[1]
    t = par[2]
    n =  nm*(b*x)**t / (1.0 + (b*x)**t)
    return n
#
# use Langmuir to estimate nm,K
#
def fFitLang(x , y):
    ix = np.ones(x.shape)
    yf = x/y
    A = np.vstack([x,ix]).T
    m,b = np.linalg.lstsq(A,yf.T)[0]
    nm = 1/m
    K = m/b
    return nm,K
#
# create typical LF linear transforms
#
def fLinLF( x,y,nm,b,t):
    yLrp = np.log(y/(nm-y))
    yLfit = t*np.log(x) + t*np.log(b)
    return yLrp,yLfit
#
nmL,Kl = fFitLang(x,y)
#
# Use Langmuir params as initial guess for RP
#
ti = 0.8
nmi = nmL
bi = Kl
par = (nmi , bi, ti)
#
p = optim.fmin(fMinLF, par, args=(x,y), maxfun=10000,disp=0)
#
# predict isotherm from LF params
#
nm = p[0]
b = p[1]
t = p[2]
yLrp,yLfit = fLinLF(x,y,nm,b,t)
yLF = fLF(p,x)
#
# 
# LF params
#
#########
#
# send data back to the MicroActive UI and Report system
#
#
def fTextOut( x, oFormat):
    tOut = [(oFormat % xOut) for xOut in x]
    return tOut
#
# Create a summary report
#
mic.summary('Summary')
mic.summary.add( 'Sips parameters', ["Nm (mmol/g) = ",
                                     'b (1/mbar) = ', 
                                     't (exponent) = '
                                     ],
                 ["%10.5f" % nm,
                  "%10.5f" % b, 
                  "%10.5f" % t 
                 ] )
# create plots
########
#
# create color table - this is a convenient table - it may not suit
# your sense of aesthetics.
#
black = "#000000"
blue = "#0000FF"
green = "#008000"
red = "#FF0000"
#
navy = "#000080"
teal = "#008080"
lime = "#00FF00"
aqua = "#00FFFF"
purple = "#800080"
gray = "#808080"
#
########
#
mic.graph('Model','Pressure, mbar','Qads, mmoles/g')
mic.graph.add('Isotherm', x , y, color = red, graphtype = 'points', marker = 'o')
mic.graph.add('Sips (Langmuir-Freundlich)', x,
              yLF , color = blue, linestyle = '-', graphtype = 'curve')
#
mic.graph('Linear Transform', 'ln[p]','ln[n/(nm-n)]', xlinear = 1, ylinear = 1)
mic.graph.add('Data', np.log(x) , yLrp,color = red, graphtype = 'points', marker = 'o')
mic.graph.add('Sips (Langmuir-Freundlich)', np.log(x),
              yLfit , color = blue , linestyle = '-', graphtype = 'curve')
#
# create tabular output
#
mic.table(title='Tabular Report')
mic.table.addtable('Tabular Report')
mic.table.addcolumn('P, mbar', fTextOut( x, '%6.4f'))
mic.table.addcolumn('Qads, mmol/g', fTextOut(y, '%8.3f'))
mic.table.addcolumn('Q(Sips), mmol/g', fTextOut(yLF, '%8.3f'))
mic.table.addcolumn('Residual, mmol/g', fTextOut((y-yLF), '%8.3f'))
mic.table.addcolumn('% Different', fTextOut((y-yLF)/y*100.0, '%8.3f'))

